//---------------------------------------------------------------------------
// jrdebug_compiled_macros.h
// main macros that are called by user
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// Macros used when debugging is compiled in
//---------------------------------------------------------------------------
//
// just used as empty expression (see JrDebugErrorIfNotCompiledIn)
#define JrDoNothing

// enabled flag - we use this check before all print functions to minimize runtime costs when compiled in but disabled
#define JrDebugEnabled JrDebug::enabled
#define JrDebugEnable(REASON) JrDebug::SetEnabled(true,REASON,__FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__, __FUNC__, __func__ )
#define JrDebugDisable(REASON) JrDebug::SetEnabled(false,REASON,__FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__, __FUNC__, __func__ )
//
// temporarily disable and call
#define JrDebugQuietCall(X) {bool enableflag=JrDebug::GetEnabled();JrDebug::SetEnabled(false);X;JrDebug::SetEnabled(enableflag);}
//
// macros that allow us to quickly disable with very little cost, even when debugging is compiled in
#define JrDebugPrintf if (!JrDebugEnabled) ; else JrDebugInternalPrintf
#define JrDebugPrintString if (!JrDebugEnabled) ; else JrDebugInternalPrintString
#define JrDebugPrintfError if (!JrDebugEnabled) ; else JrDebugInternalPrintfError
//
// ifdebug
#define JrDebugIf(X) if (X && JrDebugEnabled) 
//
// assert and verify
#define JrDebugAssert(X) { if (JrDebugEnabled) JrDebugInternalAssert( JrEnumDummyFunctionDisambiguator_Assert_DummyVal , true , (bool)(X) , #X ); }
#define JrDebugVerify(X) { if (JrDebugEnabled) JrDebugInternalAssert( JrEnumDummyFunctionDisambiguator_Assert_DummyVal, false , (bool)(X) , #X ); }
//
// replacement throw (triggers an log message of JrdException type, with severity set by static JrDebug::exceptionlevel; can be disabled by setting to JrDebugIgnoreLevel or calling JrDebug::DisableExceptionLogging())
#define JrDebugThrow(X,Y) { if (JrDebug::exceptionlevel!=JrDebugIgnoreLevel) JrDebugPrintf(JrdException,JrDebug::exceptionlevel,"%s: throwing %s",Y,#X); throw X; }
//
// stream versions
#define JrDebugStream if (!JrDebugEnabled) ; else JrDebugInternalStream
#define JrDebugStreamError if (!JrDebugEnabled) ; else JrDebugInternalStreamError
//
// debug block
#define JrDebugBlock(Nstr) JrDebugInternalDebugBlock(Nstr,true)
#define JrDebugBlockReport(Nstr) { if (JrDebugEnabled) JrDebug::DebugBlockReport(Nstr); }
#define JrDebugActivityPush(Nstr) { if (JrDebugEnabled) JrDebugInternalActivityPush(Nstr); }
#define JrDebugActivityPop(Nstr) { if (JrDebugEnabled) JrDebugInternalActivityPop(Nstr); }
//
// put debug block around a function call (note: func gets execute whether debugging is enabled or not)
#define JrDebugEnclose(X) {JrDebugBlock(#X);X;}
//
// in case you want to test and confirm that debugging is compiled in
#define JrDebugErrorIfNotCompiledIn JrDoNothing
#define JrDebugErrorIfCompiledIn JRDEBUG_ERROR_This_error_is_triggered_on_purpose_using_the_ErrorIfCompiledIn_macro.__JrDebugLogger_is_being_compiled-in(!)_remove_this_line_or_disable_compiling-in_ofJrDebugLogger
//
// unit testing
// ATTN: do we need both of these run test things?
#define JrDebugRegisterTestFunction(SimpName,Desc,FName,KString) JrDebugInternalRegisterTestFunctionM(SimpName,Desc,KString,#FName,FName, __FILE__ , __LINE__ ,  __TIMESTAMP__);
#define JrDebugRunTestFunction(SimpName,Desc,FName) JrDebugInternalRunTestFunctionM(SimpName,Desc,#FName,FName, __FILE__ , __LINE__ ,  __TIMESTAMP__)
#define JrDebugTestRun(FPOINT) { if (JrDebugEnabled) JrDebug::RunTestByPointer(FPOINT); }
#define JrDebugTest(X) { if (JrDebugEnabled) JrDebugInternalTest( JrdTestMsgContinue, (bool)(X) , #X , ""); }
#define JrDebugTestEx(X) { if (JrDebugEnabled) JrDebugInternalTest( JrdTestMsgStop, (bool)(X) , #X , "" ); }
#define JrDebugTestInfo(X,Y) { if (JrDebugEnabled) JrDebugInternalTest( JrdTestMsgContinue, (bool)(Y) , X , #Y ); }
#define JrDebugTestExInfo(X,Y) { if (JrDebugEnabled) JrDebugInternalTest( JrdTestMsgStop, (bool)(Y) , X , #Y ); }
//
// commandline
#define JrDebugParseCommandlineArgc(DESC,ARGCP,ARGV,DEFAULTCOMLINE) JrDebug::ParseCommandline(DESC,ARGCP,ARGV,"","",DEFAULTCOMLINE,"")
#define JrDebugParseCommandlineStr(DESC,ARGLINE,DEFAULTCOMLINE) JrDebug::ParseCommandline(DESC,(int*)0,(char**)0,ARGLINE,"",DEFAULTCOMLINE,"")
#define JrDebugParseCommandline(DESC,ARGCP,ARGV,ALTCOMLINESTR,PRECOMLINEOPTS,DEFAULTCOMLINE,DEFAULTOUTPUTOPTIONS) JrDebug::ParseCommandline(DESC,ARGCP,ARGV,ALTCOMLINESTR,PRECOMLINEOPTS,DEFAULTCOMLINE,DEFAULTOUTPUTOPTIONS)
#define JrDebugParseCommandlineArgString(DESC,ARG0,FOUNDARGS) JrDebug::ParseCommandlineArgString(DESC,ARG0,FOUNDARGS)
//
// misc
#define JrDebugEnableExceptionLogging(LEVEL) JrDebug::EnableExceptionLogging(LEVEL)
#define JrDebugSetDisplayPassingAsserts(BOOLVAL) JrDebug::SetDisplayPassingAsserts(BOOLVAL)
//
// announce
#define JrDebugAnnounce(APPNAME,BOOLVALL,ARGC,ARGV) JrDebug::Announce(APPNAME,BOOLVALL,ARGC,ARGV)
//
// accessors
#define JrDebugSetDefaultOutputOptions(OPSTRING) JrDebug::set_defaultoutputoptions(OPSTRING)
//
// output formats
#define JrDebugRegisterOutputFormatter(ObjName) ObjName JrDebugStringify2args(JrdUniqueId,ObjName)(true)
//
// callback func
// ATTN: we might want to use a self-registering thing like our other classes
#define JrDebugSetCallbackPointer(CALLBACKFP) JrDebug::SetCallbackPointer(CALLBACKFP)
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// Do you want to take over the calls to TRACE, ASSERT, VERIFY if debugging is enabled?
//  (this is *NOT* engaged by default)
#ifdef JrDebugDIRECTIVE_TakeoverAssertTrace
	#undef TRACE
	#undef ASSERT
	#undef assert
	#undef VERIFY
	#undef verify
	#define TRACE JrDebugPrintf
	#define ASSERT JrDebugAssert
	#define assert JrDebugAssert
	#define VERIFY JrDebugVerify
	#define verify JrDebugVerify
#endif
//---------------------------------------------------------------------------





//---------------------------------------------------------------------------
// Internal macros that create and then use operator() to engage the printing, allowing use of some other useful macro variables
// based on code from Paul Mclachlan, http://www.codeproject.com/debug/location_trace.asp#xx386001xx
#define JrDebugInternalPrintf (JrDebug( __FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__ , __FUNC__, __func__ , JrdMessage))
#define JrDebugInternalPrintfError (JrDebug( __FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__ , __FUNC__, __func__ , JrdError))
#define JrDebugInternalPrintString (JrDebug( __FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__ , __FUNC__, __func__ , JrdMessage))
#define JrDebugInternalAssert (JrDebug( __FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__ , __FUNC__, __func__ , JrdMessage))
#define JrDebugInternalTest (JrDebug( __FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__ , __FUNC__, __func__ , JrdMessage))
//
// streaming macro - we use the () to solve an interesting issue (see methods for more info)
#define JrDebugInternalStream (JrDebugs( __FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__ , __FUNC__, __func__ )())
#define JrDebugInternalStreamError (JrDebugs( __FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__ , __FUNC__, __func__ , JrdError)())
// debug blocks macro
#ifdef __COUNTER__
	#define JrdUniqueId __COUNTER__
#else
	#define JrdUniqueId __LINE__
#endif
#define JrDebugStringify2args(x,y) x ## y
#define JrDebugStringify3args(x,y,z) x ## y ## z
#define JrDebugStringify4args(w,x,y,z) w ## x ## y ## z
#define JrDebugInternalDebugBlock2If(Nstr, IsScopedelete, J, F,L,T,U,U2,U3) JrDebugBlockObject JrDebugStringify2args(jrdscopedobj_, J); if (!JrDebugEnabled) ; else JrDebugStringify2args(jrdscopedobj_, J).init(Nstr,F,L,T,U,U2,U3,IsScopedelete)
#define JrDebugInternalDebugBlock(Nstr,IsScopedelete) JrDebugInternalDebugBlock2If(Nstr, IsScopedelete, JrdUniqueId, __FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__, __FUNC__, __func__ )
//
#define JrDebugInternalActivityPush(Nstr) if (JrDebugEnabled) JrDebug::PushActivity(Nstr,JrdNote,NULL,__FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__, __FUNC__, __func__ )
#define JrDebugInternalActivityPop(Nstr) if (JrDebugEnabled)JrDebug::PopActivity(Nstr,JrdNote,NULL,__FILE__ , __LINE__ ,  __TIMESTAMP__ , __FUNCTION__, __FUNC__, __func__ )
// create an anonymous object
#define JrdRegTestHelper2(x,y) JrDebugStringify2args(x,y)
#define JrdRegTestHelper3(x,y,z) JrDebugStringify3args(x,y,z)
//
#define JrDebugInternalRegisterTestFunctionM(SimpName,Desc,KString,FName,FPointer,Fi,Li,Ti) JrDebugInternalRegisterTestFunctionM2( SimpName, Desc,KString,FName,FPointer, Fi,Li,Ti)
#define JrDebugInternalRegisterTestFunctionM2(SimpName, Desc,KString,FName,FPointer,Fi,Li,Ti) JrDebugRegisteredTestObj JrdRegTestHelper2( JrdTstObj_ , JrdUniqueId ) (1,SimpName,Desc,KString,FName,FPointer, Fi , Li ,  Ti )
//
#define JrDebugInternalRunTestFunctionM(SimpName, Desc,FName,FPointer,Fi,Li,Ti) JrDebugInternalRunTestFunctionM2( SimpName, Desc,FName,FPointer, Fi,Li,Ti)
#define JrDebugInternalRunTestFunctionM2(SimpName,Desc,FName,FPointer,Fi,Li,Ti) { JrDebugRegisteredTestObj temprtop(SimpName,Desc,"manualadd",FName,FPointer, Fi , Li ,  Ti ); JrDebug::RunTestInternal(&temprtop,true);}
//---------------------------------------------------------------------------

