//---------------------------------------------------------------------------
// jrdebug_output_formatters_jrd.h
// derived methods for different outputs
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// Jrd (internal custom style)
// register the factory
JrDebugRegisterOutputFormatter(JrDebugRegisteredOutputFormatter_Jrd);
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// constructor and destructor
JrDebugRegisteredOutputFormatter_Jrd::JrDebugRegisteredOutputFormatter_Jrd(bool isfactory)
	:JrDebugRegisteredOutputFormatter()
{
	// register if appropraite
	if (isfactory)
		{
		// create a NEW object for registering, dont try to attach this global one
		JrDebugSingletonInsurer dummy(false);
		JrDebug::RegisterFormatterFactory(MakeNew());
		return;
		}
}
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// shutdown and startup
bool JrDebugRegisteredOutputFormatter_Jrd::Startup(const char *optionstring)
{
	// process optionstring
	if (strcmp(optionstring,"")!=0)
		return false;
	// return success
	return true;
}

bool JrDebugRegisteredOutputFormatter_Jrd::Shutdown()
{
	// virtual shut down (any open files will be closed by caller when we return)
	return true;
}
//---------------------------------------------------------------------------



//---------------------------------------------------------------------------
// HandleDebugMessage should format the user's string and call DoOutput one or more times
// Note it will be invoked with any user % args already repalced
void JrDebugRegisteredOutputFormatter_Jrd::HandleDebugMessage(JrDebug *jrd,const char *str)
{
	// Format for special jrd mode

	// buffers with limited size (we could use a string here to limit this to only one big buffer)
	char buf1[JrDebug_MaxBufferSize];
	char buf2[JrDebug_MaxBufferSize];

	// copy user string into internal temp buffer
	strncpy(buf2,str,JrDebug_MaxBufferSize-1);
	buf2[JrDebug_MaxBufferSize-1]='\0';

	// add activitymodstring to currentactivitystring
	jrd_stdstringT currentactivitystring=jrd->get_currentactivitystring();
	jrd_stdstringT activitymodstring=jrd->get_activitymodstring();
	jrd_stdstringT astring="";
	jrd_stdstringT iastring=currentactivitystring;
	if (iastring=="" && activitymodstring!="")
		iastring=jrd_stdstringT(activitymodstring);
	else if (activitymodstring!="")
		iastring+=jrd_stdstringT(".")+activitymodstring;	
	if (iastring!="")
		astring=" |*Activity*| "+iastring;

	// get current time
	time_t timenum;
	time(&timenum);

	// function name is present?
	jrd_stdstringT functionnamestring = jrd->get_functionnamestring();
	jrd_stdstringT functionnameaddstring;
	if (functionnamestring!="")
		functionnameaddstring=jrd_stdstringT("|*Func*| ")+functionnamestring+jrd_stdstringT(" ");
	else
		functionnameaddstring="";

	// file info
	jrd_stdstringT filedirstring = jrd->get_filedirstring();
	jrd_stdstringT filenamestring = jrd->get_filenamestring();

	// misc
	jrd_stdstringT messagetypestring = jrd->get_messagetypestring();
	jrd_stdstringT timestampstring = jrd->get_timestampstring();
	long linenumber=jrd->get_linenumber();
	long runid=jrd->get_runid();
	int severitylevelnumber=jrd->get_severitylevelnumber();
	long debugindex=jrd->get_debugindex();

	// JRD FORMAT
	// ok now make our strings and use a placeholder where we want to printf'd argreplaced input
	char formatplaceholder[]="__FMPlH__";
	// construct the format string to display (uses snprintf for safety to insure we dont go over buffer size)
	mysnprintf(buf1, JrDebug_MaxBufferSize-1, "|*RunId*| %ld |*TimeStamp*| %ld |*Index*| %ld |*Type*| %s |*Level*| %d%s |*File*| %s |*FilePath*| %s |*FileDate*| %s |*Line*| %d %s|*Message*| %s", \
		runid, (long)timenum, debugindex, messagetypestring.c_str(),severitylevelnumber, astring.c_str(), filenamestring.c_str() , filedirstring.c_str(), timestampstring.c_str(), linenumber, functionnameaddstring.c_str() , formatplaceholder );
	buf1[JrDebug_MaxBufferSize-1] = 0;

	// now replace format into buf1 using placeholder
	char *ppos = strstr(buf1,formatplaceholder);
	if (ppos!=0)
		{
		if (strlen(buf1)+strlen(buf2)>=JrDebug_MaxBufferSize+3)
			{
			// make sure its not too big
			buf2[(strlen(buf1)+strlen(buf2))-(JrDebug_MaxBufferSize+3)]='\0';
			}
		*ppos='\0';
		strcat(buf2,ppos+strlen(formatplaceholder));
		strcat(ppos,buf2);
		}

	// add CR
	strcat(buf1,"\n");

	// now its ready for output
	DoOutput(buf1);
}
//---------------------------------------------------------------------------
